// Example.cp

// includes
#include "Example.h"

// constants
const char pWindowFrame[] = "Window Frame";
const char pText[] = "Text";
const BRect kAboutButtonRect(20, 20, 220, 50);
const BRect kTextFieldRect(20, 70, 300, 200);
const char kDefaultText[] = "This is the default text.\n\nIt is stored in the preferences file";
const BRect kDefaultWindowFrame(20,80,300,300);

// globals
ExamplePrefs *gPrefs = NULL;

//
// main
//
main()
{
	ExampleApp app;
	app.Run();
}

//
// ExampleApp::ExampleApp
//
ExampleApp::ExampleApp()
	: BApplication("application/x-be-executable")
{
	gPrefs = new ExamplePrefs();
	gPrefs->Open();
	
	BRect r;
	gPrefs->FindRect(pWindowFrame,&r);
	window = new ExampleWindow(r);
	window->Show();
}


//
// ExampleApp::Quit
//
void ExampleApp::Quit()
{
	gPrefs->Close();		// is nice to close it, isn't it?
	delete gPrefs;			// this will close it if we forget, though
	gPrefs = NULL;			// and it doesn't matter after this anyway
	
	inherited::Quit();
}


//
// ExamplePrefs::AddDefaultData
//
void ExamplePrefs::AddDefaultData()
{
	AddRect(pWindowFrame, kDefaultWindowFrame);
	AddData(pText, B_RAW_TYPE, kDefaultText, strlen(kDefaultText), false);
}


//
// ExampleWindow::ExampleWindow
//

ExampleWindow::ExampleWindow(BRect r)
	: BWindow(r, "Preferences Example", B_DOCUMENT_WINDOW, B_NOT_ZOOMABLE)
{
	Lock();
	
	BMessage *msg = new BMessage(B_ABOUT_REQUESTED);
	fAboutBtn = new BButton(kAboutButtonRect, "About", "About...", msg);
	fAboutBtn->SetTarget(be_app);
	AddChild(fAboutBtn);
	
	BRect r2 = kTextFieldRect;
	r2.OffsetBy(-r2.left, -r2.top);
	fText = new BTextView(kTextFieldRect, "Text", r2, 
					B_FOLLOW_LEFT_RIGHT|B_FOLLOW_TOP_BOTTOM,
					B_NAVIGABLE|B_WILL_DRAW|B_PULSE_NEEDED);
	char *text;
	ssize_t size;
	gPrefs->FindData(pText, B_RAW_TYPE, &text, &size);
	fText->SetText(text,size);
	AddChild(fText);
	
	Unlock();
}

//
// ExampleWindow::QuitRequested
//
bool ExampleWindow::QuitRequested()
{
	// save the preferences stuff from this window
	BRect f;
	f = Frame();
	gPrefs->ReplaceRect(pWindowFrame,f);
	
	const char *text = fText->Text();
	ssize_t size = fText->TextLength();
	gPrefs->ReplaceData(pText, B_RAW_TYPE, text, size);
	
	// post a quit message to the application
	be_app->PostMessage(B_QUIT_REQUESTED);
	
	// and yes we can quit
	return true;
}
